/*

originally written by Christian, cptbjorn@gmail.com
DS18B20 Temp Sensor and reformat for 20x4 LCD added by Tom, droidninjas@gmail.com
optimized and modified for more dim steps, better Temperature readings, possibility for changing PWM Frequency for Pin 3 and 11, and red LEDs including by Volker, Volkermi@gmx.de

*/

#include <OneWire.h>
#include <DallasTemperature.h>
#include "Wire.h" 
#define DS1307_I2C_ADDRESS 0x68 //set rtc
#include <LiquidCrystal.h> // initialize the library with the numbers of the interface pins
#define ONE_WIRE_BUS 8 //Define the pin of the DS18B20


/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  R E L A Y   P A R T  |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  S I M P L E   O N   A N D   O F F   F E A T U R E |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/



const int ledPin1 =  A2;          // pin number for relay 1 was digital pin 2
const int ledPin2 =  A3;          // pin number for relay 2 was digital pin 8


int ledState1 = LOW;             
int ledState2 = LOW; 
long previousMillis1 = 0;        
long previousMillis2 = 0;
long interval1 = 30000;          // interval at which to blink (milliseconds) for RELAY1
long interval2 = 50000;		 // interval at which to blink (milliseconds) for RELAY2


/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  L E D   D I M M I N G   P A R T  |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  F A D E S   I N   A N D   O U T  |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/



int blueramptime = 30 ;    // time for blue LEDs to dim on and off in minutes: default 60
int whiteramptime = 60 ;  // time for white LEDs to dim on and off in minutes: default 90
int bluemin = 0 ;          // minimmum dimming value of blue LEDs, range of 0-255
int bluemax = 255 ;        // maximum dimming value of blue LEDs, range of 0-255
int whitemin = 0 ;         // minimum dimming value of white LEDs, range of 0-255
int whitemax = 170 ;       // maximum dimming value of white LEDs, range of 0-255
int redmin = 0 ;           // maximum dimming value of red LEDs, range of 0-255
int redmax = 255 ;         // maximum dimming value of red LEDs, range of 0-255
int photoperiod = 540 ;    // amount of time array is on at full power in minutes: default 420
int ontime = 10 ;          // time of day (hour, 24h clock) to begin photoperiod fade in
int blue = 3;              // blue LEDs connected to digital pin 3 (pwm)
int white = 11;            // white LEDs connected to digital pin 11 (pwm)
int red = 9;            // red LEDs connected to digital pin 9 (pwm)
int fan = 2;            // Fan control connected to digital pin 2
int fuge = A1;            // fuge light power relay connected to analog pin 1
int moon = 10;            // moon light connected to digital pin 10 (pwm)

int iBlueIntensity;    //declare the integer of blue intensity
float fBlueIntensity;    //declare the floating point version of blue intensity


//Note: The white LEDs are not full powered. If you need full power you have to set the end value to 255!
int bluepercent[101] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 75, 80, 85, 90, 95, 100, 105, 110, 115, 120, 125, 130, 135, 140, 145, 150, 155, 160, 165, 170, 175, 180, 185, 190, 195, 200, 205, 210, 215, 255 };   // this line is needed if you are using meanwell ELN60-48D
int whitepercent[101] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 90, 95, 100, 105, 110, 115, 120, 125, 130, 135, 140, 150, 155, 160, 170 };   // these are the values in 1% increments

//int bluepercent[11] = { 0, 26, 52, 78, 103, 128, 154, 180, 205, 230, 255 };   // this line is needed if you are using meanwell ELN60-48P
//int whitepercent[11] = { 0, 26, 52, 78, 103, 128, 154, 180, 205, 230, 255 };   // these are the values in 10% increments

//int pwm_one = 9;        // extra pwm pin for future use

//LiquidCrystal lcd(13, 12, 7, 6, 5, 4); //My diy circuit layout use this if you copy my board
//LiquidCrystal lcd(11, 12, 13, 14, 15, 16); //Teensy++ 2.0 alternate to Arduino boards
LiquidCrystal lcd(12, 13, 4, 5, 6, 7);   // typically 8, 9, 4, 5, 6, 7
                                         // have to change to free up more pwm pins

//  Set up the custom fish icon looking right
byte newChar[8] = {
	B00000,
	B00000,
	B10001,
	B11011,
	B11111,
	B11111,
	B11001,
	B10000
};

byte newChar1[8] = {
	B00001,
	B00010,
	B11101,
	B11010,
	B11111,
	B11110,
	B11100,
	B00000
};
//  Set up the custom fish icon looking left
byte newChar2[8] = {
	B00000,
	B00000,
	B10001,
	B11011,
	B11111,
	B11111,
	B10011,
	B00001
};
byte newChar3[8] = {
	B10000,
	B01000,
	B10111,
	B01011,
	B11111,
	B01111,
	B00111,
	B00000
};
//  Set up the custom sun icon
byte newChar4[8] = {
	B00111,
	B01110,
	B11100,
	B11000,
	B11000,
	B11100,
	B01110,
	B00111
};
byte newChar5[8] = {
	B01010,
	B10000,
	B00010,
	B01001,
	B00100,
	B10000,
	B00100,
	B10001
};
//  Set up the custom moon icon
byte newChar6[8] = {
	B01010,
	B00100,
	B00001,
	B11011,
	B00011,
	B00001,
	B01000,
	B10010
};
byte newChar7[8] = {
	B10010,
	B00100,
	B10000,
	B11011,
	B11000,
	B10000,
	B00010,
	B01001
};

// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);

// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);

/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  R T C   C L O C K   D S 1 3 0 7  |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/



byte decToBcd(byte val)    // Convert normal decimal numbers to binary coded decimal
{
  return ( (val/10*16) + (val%10) );
}


byte bcdToDec(byte val)    // Convert binary coded decimal to normal decimal numbers
{
  return ( (val/16*10) + (val%16) );
}

// 1) Sets the date and time on the ds1307
// 2) Starts the clock
// 3) Sets hour mode to 24 hour clock
// Assumes you're passing in valid numbers
void setDateDs1307(byte second, // 0-59
byte minute, // 0-59
byte hour, // 1-23
byte dayOfWeek, // 1-7
byte dayOfMonth, // 1-28/29/30/31
byte month, // 1-12
byte year) // 0-99


{
  Wire.beginTransmission(DS1307_I2C_ADDRESS);
  Wire.send(0);
  Wire.send(decToBcd(second)); // 0 to bit 7 starts the clock
  Wire.send(decToBcd(minute));
  Wire.send(decToBcd(hour));   // If you want 12 hour am/pm you need to set
  // bit 6 (also need to change readDateDs1307)
  Wire.send(decToBcd(dayOfWeek));
  Wire.send(decToBcd(dayOfMonth));
  Wire.send(decToBcd(month));
  Wire.send(decToBcd(year));
  Wire.endTransmission();
}

// Gets the date and time from the ds1307
void getDateDs1307(byte *second,
byte *minute,
byte *hour,
byte *dayOfWeek,
byte *dayOfMonth,
byte *month,
byte *year)
{
  // Reset the register pointer
  Wire.beginTransmission(DS1307_I2C_ADDRESS);
  Wire.send(0);
  Wire.endTransmission();

  Wire.requestFrom(DS1307_I2C_ADDRESS, 7);

  // A few of these need masks because certain bits are control bits
  *second = bcdToDec(Wire.receive() & 0x7f);
  *minute = bcdToDec(Wire.receive());
  *hour = bcdToDec(Wire.receive() & 0x3f); // Need to change this if 12 hour am/pm
  *dayOfWeek = bcdToDec(Wire.receive());
  *dayOfMonth = bcdToDec(Wire.receive());
  *month = bcdToDec(Wire.receive());
  *year = bcdToDec(Wire.receive());
}

int moonPhase(int moonYear, int moonMonth, int moonDay)
{
  int dayFromYear, dayFromMonth;
  double julianDay;
  int phase;

  if (moonMonth < 3)            //keep the month before march
  {
    moonYear--;            //take away a year
    moonMonth += 12;        //add an extra 12 months (the year taken away from before)
  }
  ++moonMonth;
  dayFromYear = 365.25 * moonYear; //get days from current year
  
  
  dayFromMonth = 30.6 * moonMonth; //get number of days from the current month
  
  julianDay = dayFromYear + dayFromMonth + moonDay - 694039.09; //add them all  
  julianDay /= 29.53;        //divide by the length of lunar cycle
 
  phase = julianDay;        //take integer part
  julianDay -= phase;        //get rid of the int part
 
  phase = julianDay*8 + 0.5;    //get it between 0-8 and round it by adding .5
  phase = phase & 7;        //get a number between 1-7
  
  return phase;         //1 == new moon, 4 == full moon
}

/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  P W M   F R E Q U E N C Y |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
/*
Base frequencies:
 *      o The base frequency for pins 3, 9, 10, and 11 is 31250 Hz.
 *      o The base frequency for pins 5 and 6 is 62500 Hz.
 *   - Divisors:
 *      o The divisors available on pins 5, 6, 9 and 10 are: 1, 8, 64,
 *        256, and 1024.
 *      o The divisors available on pins 3 and 11 are: 1, 8, 32, 64,
 *        128, 256, and 1024.
*/

void setPwmFrequency(int pin, int divisor) {
  byte mode;
  if(pin == 5 || pin == 6 || pin == 9 || pin == 10) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 64: mode = 0x03; break;
      case 256: mode = 0x04; break;
      case 1024: mode = 0x05; break;
      default: return;
    }
    if(pin == 5 || pin == 6) {
      TCCR0B = TCCR0B & 0b11111000 | mode;
    } else {
      TCCR1B = TCCR1B & 0b11111000 | mode;
    }
  } else if(pin == 3 || pin == 11) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 32: mode = 0x03; break;
      case 64: mode = 0x04; break;
      case 128: mode = 0x05; break;
      case 256: mode = 0x06; break;
      case 1024: mode = 0x07; break;
      default: return;
    }
    TCCR2B = TCCR2B & 0b11111000 | mode;
  }
}


/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  O N E S E C O N D |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/



void onesecond() //function that runs once per second while program is running
{
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  getDateDs1307(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);

  lcd.setCursor(0, 1);
  lcd.print(hour, DEC);
//  if(hour>0)
//  {
//    if(hour<=12)
//    {
//      lcd.print(hour, DEC);
//    }
//    else
//    {
//      lcd.print(hour-12, DEC);
//    }
//  }
//  else
//  {
//    lcd.print("12");
//  }
  lcd.print(":");
  if (minute < 10) {
    lcd.print("0");
  }
  lcd.print(minute, DEC);
  //lcd.print(":");
  //if (second < 10) {
  //  lcd.print("0");
  //}
  //lcd.print(second, DEC);
//  if(hour<12)
//  {
//    lcd.print("am");
//  }
//  else
//  {
//    lcd.print("pm");
//  }
  lcd.print(" ");
  delay(1000);
}


/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  F A N S   O N |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void FansOn()
{
  digitalWrite(fan, HIGH);  
  lcd.setCursor(17, 3);
  lcd.print("On ");
  //analogWrite(fan, 255);
    
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  F A N S   O F F |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void FansOff()
{
  digitalWrite(fan, LOW);
  lcd.setCursor(17, 3);
  lcd.print("Off");
    //analogWrite(fan, 0);
    
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  F U G E  L I G H T  O N |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void FugeOn()
{
  digitalWrite(fuge, HIGH);  
  lcd.setCursor(7, 3);
  lcd.print("On ");
  //analogWrite(fuge, 255);
    
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  F U G E  L I G H T   O F F |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void FugeOff()
{
  digitalWrite(fuge, LOW);
  lcd.setCursor(7, 3);
  lcd.print("Off");
    //analogWrite(fuge, 0);
    
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  M O O N  L I G H T  O N |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void MoonOn()
{
  //digitalWrite(Moon, HIGH);  
  lcd.setCursor(6, 1);
  lcd.write(4);
  lcd.write(5);
  //analogWrite(moon, 255);
    
}
/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  M O O N  L I G H T   O F F |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
void MoonOff()
{
  //digitalWrite(Moon, LOW);
  lcd.setCursor(6, 1);
  lcd.write(6);
  lcd.write(7);
   //analogWrite(moon, 0);
    
}

/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  R E L A Y 1 |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/




void relay1()  //FUNCTION TO TURN ON AND OFF RELAY 1.
{ 
  unsigned long currentMillis = millis();

  if(currentMillis - previousMillis1 > interval1) 
  { 
    previousMillis1 = currentMillis;   
    if (ledState1 == LOW)
      ledState1 = HIGH;
    else
      ledState1 = LOW;
    digitalWrite(ledPin1, ledState1);
  }
}



/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  R E L A Y 2 |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/




void relay2()
{
  unsigned long currentMillis2 = millis();

  if(currentMillis2 - previousMillis2 > interval2) 
  {
    previousMillis2 = currentMillis2;   
    if (ledState2 == LOW)
      ledState2 = HIGH;
    else
      ledState2 = LOW;
    digitalWrite(ledPin2, ledState2);
  }
}

/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  D E F I N E  :  T E M P E R A T U R E |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/

void requestTemperature()
{
  // DS18B20 display
  sensors.requestTemperatures(); // Send the command to get temperatures
        // set the cursor to column 0, line 1
      // (note: line 1 is the second row, since counting begins with 0):
      lcd.setCursor(13, 2);
      
      float temp1=0, temp2=0;
    
      //lcd.print("Tank Temp:");
      temp1= sensors.getTempCByIndex(1);
      lcd.print(sensors.getTempCByIndex(1)); 
      lcd.print((char)223);
      lcd.print("C");
     
      lcd.setCursor(2, 2);
      //lcd.print("LED Temp: ");
      temp2= sensors.getTempCByIndex(0);
      lcd.print(sensors.getTempCByIndex(0)); 
      lcd.print((char)223);
      lcd.print("C");
      
      if ( (temp1) >= 44)
        {
          digitalWrite(fan, HIGH);
          lcd.setCursor(17, 3);
          lcd.print("On ");
        }
      if ( (temp1) < 40)
        {
          digitalWrite(fan, LOW);
          lcd.setCursor(17, 3);
          lcd.print("Off");
        }
       
//      if ( (temp2) >= 44)
//        {
//          digitalWrite(fan, HIGH);
//          lcd.setCursor(17, 3);
//          lcd.print("On ");
//        }
//      if ( (temp2) < 40)
//        {
//          digitalWrite(fan, LOW);
//          lcd.setCursor(17, 3);
//          lcd.print("Off");
//        }
}



/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  S E T U P  |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/


void setup() {
  pinMode(ledPin1, OUTPUT);    // set the digital pin as output:
  pinMode(ledPin2, OUTPUT);    // set the digital pin as output:
  pinMode(fan, OUTPUT);      // Set analog pin 0 as a output
  pinMode(fuge, OUTPUT);      // Set analog pin 1 as a output
  pinMode(moon, OUTPUT);      // Set ditital pin 2 as a output
  sensors.begin();             // Start up the DS18B20 Temp library
  
  Serial.begin(115200);
  Serial.println("Hello.");
  
  // Set pin 3's PWM frequency to 118 Hz (31250/265 = 118)
  // Note that the base frequency for pins 3, 9, 10, and 11 is 31250 Hz
  setPwmFrequency(3, 256);
  
  // Set pin 11's PWM frequency to 118 Hz (31250/265 = 118)
  // Note that the base frequency for pins 3, 9, 10, and 11 is 31250 Hz
  setPwmFrequency(11, 256);

  /*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  S E T U P - D I S P L A Y |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/



  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  Wire.begin();

  // Change these values to what you want to set your clock to.
  // You probably only want to set your clock once and then remove
  // the setDateDs1307 call.
  second = 00;
  minute = 36 ;
  hour = 21;
  dayOfWeek = 2;  // Sunday is 0
  dayOfMonth = 15;
  month = 01;
  year = 13;
  
  //Use the next line for setting the clock once
  //setDateDs1307(second, minute, hour, dayOfWeek, dayOfMonth, month, year);

  analogWrite(blue, bluemin);
  analogWrite(white, whitemin);
  analogWrite(red, redmin);
  lcd.createChar(0, newChar);
  lcd.createChar(1, newChar1);
  lcd.createChar(2, newChar2);
  lcd.createChar(3, newChar3);
  lcd.createChar(4, newChar4);
  lcd.createChar(5, newChar5);
  lcd.createChar(6, newChar6);
  lcd.createChar(7, newChar7);
  lcd.begin(20, 4); // set up the LCD's number of rows and columns: 
  lcd.setCursor(0, 0);
  lcd.write(0);
  lcd.write(1);
  lcd.setCursor(4, 0);// set the cursor to column 0, line 0
  lcd.print("Volkers Riff");
  lcd.setCursor(18, 0);
  lcd.write(3);
  lcd.write(2);
  //lcd.setCursor(7, 1);
  //lcd.print("M:");
  lcd.setCursor(9, 1);
  lcd.print("B:");
  lcd.print(33*bluemin/85);
  lcd.setCursor(15, 1);
  lcd.print("W:");
  lcd.print(33*whitemin/85);
  lcd.setCursor(11, 2);
  lcd.print("T: ");
  lcd.setCursor(0, 2);
  lcd.print("T: ");
  lcd.setCursor(1, 3);
  lcd.print("Mond: ");
  lcd.setCursor(12, 3);
  lcd.print("Fan: ");
 
}




/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  L O O P |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/




void loop()
{
  onesecond();
  relay2();
  relay1();

/*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  L U N A R |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
 
  float fSecond, fHour, fMinute;        //turn the times read from the RTC into float for math ops
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year; //declare variables to hold the times
  getDateDs1307(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year); //read the RTC times
  int daybyminute = ((hour * 60) + minute); //converts time of day to a single value in minutes
 

  fSecond = (float) second;            //sets fSecond as the float version of the integer second from the RTC
  fMinute = (float) minute;            //same as above, but for the minute
  fHour = (float) hour;                //ditto, but for the hour
  int lunarCycle = moonPhase(year, month, dayOfMonth); //get a value for the lunar cycle
  //--------MOON LED SETUP----------//

  int hour1 = hour;
  int minute1 = minute;
  //if ( ((hour == 7) && (minute < 30)) && (hour > 19))//Off at 730am.
//  Serial.print("hour: ");
//  Serial.print("  ");
//  Serial.print(hour, DEC);
//  Serial.print("  ");
  if ( ((hour == 7) && (minute < 30)) || (hour < 7))//Off at 730am.
  {
//    Serial.print("Here 1");
//    Serial.print("  ");
    fBlueIntensity = 10;        //...then we want the blue LED at the max brightness (255) old! Now 10
  }

  else if (hour > 17) //&& (hour < 19))             // On at 5pm
  {    
//    Serial.print("Here 2");
//    Serial.print("  ");
    fBlueIntensity = 10 * (((fHour - 17) + (fMinute / 59)) / 7);   //...set intensity out of 10 (255) based on time since 17:00
  }
  else
  {
//    Serial.print("Here 3");
//    Serial.print("  ");
    fBlueIntensity = 0;
  }

// Serial.print("fBlueIntensity1: ");
// Serial.print("      ");
// Serial.print(fBlueIntensity);
// Serial.print("      ");
 //---------account for the moon cycle with the blue LEDs---------//

  if (lunarCycle == 0)        //new moon
  {
    lcd.setCursor(7, 3);
    lcd.print("Neu");
    fBlueIntensity /= 2;
  }
  if ((lunarCycle == 1) || (lunarCycle == 7))    //cresent
  {
    lcd.setCursor(7, 3);
    lcd.print("1/3 ");
    fBlueIntensity /= 1.75 ;
  }
  if ((lunarCycle == 2) || (lunarCycle == 6))    //half moon
  {
    lcd.setCursor(7, 3);
    lcd.print("Halb");
    fBlueIntensity /= 1.5;
  }
  if ((lunarCycle == 3) || (lunarCycle == 5))    //gibbous
  {
    lcd.setCursor(7, 3);
    lcd.print("3/4 ");
    fBlueIntensity /= 1.25;
  }
  if (lunarCycle == 4)          //full moon is full intensity
  {
    lcd.setCursor(7, 3);
    lcd.print("Voll");
    fBlueIntensity;
  }
  


  //----------FLOAT TO INT-------------//
  iBlueIntensity = (int) fBlueIntensity;



  //---prepare the intensities to be written to pin----//

  if (iBlueIntensity < 0) //if the blue intensity is less then 0, set it to 0
  {
    iBlueIntensity = 0;
  }
// Serial.print("lunarCycle: ");
// Serial.print("      ");
// Serial.print(lunarCycle);
// Serial.print("      ");
// Serial.print("fBlueIntensity: ");
// Serial.print("      ");
// Serial.println(fBlueIntensity);


  //------------and finally, we write the values of the lights to the pins--------------//
  analogWrite(moon, iBlueIntensity);


  /*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  L O O P - D I M   F U N C T I O N |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
      

  int bluerampup;
     if (daybyminute >= (ontime*60)) 
       bluerampup = (((ontime*60) + blueramptime) - daybyminute);
     else
       bluerampup = blueramptime;
       
  int whiterampup;
    if (daybyminute >= (ontime*60 + blueramptime)) 
       whiterampup = (((ontime*60) + blueramptime + whiteramptime) - daybyminute);
     else
       whiterampup = whiteramptime;

  int whiterampdown;
    if (((ontime * 60) + photoperiod + blueramptime + whiteramptime) <= daybyminute)
      whiterampdown = (((ontime*60) + photoperiod + blueramptime + 2*whiteramptime) - daybyminute);
    else
      whiterampdown = whiteramptime;
      
  int bluerampdown;
    if (((ontime * 60) + photoperiod + blueramptime + 2*whiteramptime) <= daybyminute)
      bluerampdown = (((ontime*60) + photoperiod + 2*blueramptime + 2*whiteramptime) - daybyminute);
    else
      bluerampdown = blueramptime;






  /*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  L O O P - F A D E  I N |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/


 if (daybyminute >= (ontime*60))
  { 
    if (daybyminute <= ((ontime*60) + blueramptime + (whiteramptime/10*9))) //if time is in range of fade in, start fading in + (whiteramptime/10*9)
    {
      // Turn the fans on.
      //FansOn();
      // Turn the fuge off.
      //FugeOff();
      // Turn the moon lights off.
      MoonOff();
      // fade blue LEDs in from min to max.
      
      for (int i = 1; i <= 100; i++) // setting ib value for 10% increment. Start with 0% 
      { 
        analogWrite(blue, bluepercent[i]);
        analogWrite(red, bluepercent[i]);
        lcd.setCursor(11, 1);
        lcd.print(i);
        lcd.print(" "); 
      
        int countdown = ((bluerampup*60)/100); // calculates seconds to next step
   
        while (countdown>0)
          {
          onesecond(); // updates clock once per second
          countdown--;
          relay2();
          relay1();
          requestTemperature();
        }
      }      

      // fade white LEDs in from min to max.
      for (int i = 1; i <= 100; i++) // setting i value for 10% increment. Start with 0%
      { 
        analogWrite(white, whitepercent[i]); 
        lcd.setCursor(17, 1);
        lcd.print(i);
        lcd.print(" "); 

        int countdown = ((whiterampup*60)/100); // calculates seconds to next step
        while (countdown>0)
        {
          onesecond(); // updates clock once per second
          countdown--;
          relay2();
          relay1();
          requestTemperature();
        }
      } 
    }
  }


  /*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  L O O P - M A X  V A L U E |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/



 if (daybyminute >= ((ontime * 60) + blueramptime + whiteramptime)) 
  { 
    if ( daybyminute < ((ontime * 60) + blueramptime + whiteramptime + photoperiod)) // if time is in range of photoperiod, turn lights on to maximum fade value
    {
      // Turn the fans on.
      //FansOn();
      // Turn the fuge off.
      //FugeOff();
      // Turn the moon lights off.
      MoonOff();
      
      analogWrite(blue, 255);
      analogWrite(red, 255);
        lcd.setCursor(11, 1);
        lcd.print(100);
        lcd.print(" ");
      analogWrite(white, 170); 
        lcd.setCursor(17, 1);
        lcd.print(100);
        lcd.print(" "); 
      
    } 
  }



  /*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  L O O P - F A D E  O U T |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/



  if (((ontime * 60) + photoperiod + blueramptime + whiteramptime) <= daybyminute)
  { 
    
    if (((ontime * 60) + photoperiod + whiteramptime + 2*blueramptime + (blueramptime/10*9)) >= daybyminute)
    {
      // Turn the fans on.
      //FansOn();
      // Turn the fuge off.
      //FugeOff();
      // Turn the moon lights off.
      MoonOff();
      
      // fade white LEDs out from max to min in increments of 1 point:
      for (int i = 100; i >= 0; i--) // setting i value for 10% increment. Start with 10%
      { 
        analogWrite(blue, 255);
        analogWrite(red, 255);
        lcd.setCursor(11, 1);
        lcd.print(100);
        lcd.print(" "); 
        
        analogWrite(white, whitepercent[i]); 
        lcd.setCursor(17, 1);
        lcd.print(i);
        lcd.print(" ");  

        int countdown = ((whiterampdown*60)/100); // calculates seconds to next step
        while (countdown>0)
        {
          onesecond(); // updates clock once per second
          countdown--;
          relay2();
          relay1();
          requestTemperature();
        }

      } 

      // fade blue LEDs out from max to min in increments of 1 point:
      for (int i = 100; i >= 0; i--) // setting i value for 10% increment. Start with 10%
      { 
        analogWrite(blue, bluepercent[i]);
        analogWrite(red, bluepercent[i]);
        lcd.setCursor(11, 1);
        lcd.print(i);
        lcd.print(" "); 

        int countdown = ((bluerampdown*60)/100); // calculates seconds to next step
        while (countdown>0)
        {
          onesecond(); // updates clock once per second
          countdown--;
          relay2();
          relay1();
          requestTemperature();
        }
      }
    }
  }

// DS18B20 display
requestTemperature();

   
//  lcd.print(sensors.getTempCByIndex(0));
//  lcd.print((char)223); 
//  lcd.print("C");
  
  //delay(500);
  //*|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  Night Time |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/
//FansOff();
//FugeOn();

if (((ontime * 60) + photoperiod + (2 * blueramptime) + (2 * whiteramptime)) < daybyminute)
  {         
      
    //FansOff();
    //FugeOn();
    MoonOn();
  }
  
// if (daybyminute < (ontime*60))
//{
//
//    FansOff();
//  
//}
  
}  // END LOOP

